package Torus::Core;

use base qw( 
  DesignPattern::Object
  DesignPattern::Object::Logger
);

use strict;
use warnings;

use loadConfig;
#use Torus::Driver::ldap;
use Data::Transfer::Sync;

my $logger = Log::Dispatch::Config->instance;


my $DEBUGLEVEL = $config->get("debug_level");
my $TRACELEVEL = $config->get("trace_level");
my $bool_started;


sub addItem {

  my $torusObjectType = shift;
  my $mapiEntry = shift;
  my $basedn = shift;
  my $ldapEntry;

  if (!$bool_started) {
    Torus::Driver::ldap::connectStore();
  }

  $bool_started = 1;

  if ($DEBUGLEVEL > 3) {
    print STDOUT "=" x 3, "   ", __PACKAGE__, "   ", "=" x 30, "\n";
    print STDOUT "recieved torus-object (type=$torusObjectType)", "\n";
    print STDOUT "trying to write to (basedn='$basedn')", "\n";
  }

  if ($torusObjectType eq 'MapiContact') {
    $basedn = Torus::Driver::ldap::createOuSafe($basedn);
    my $entry = Torus::Driver::ldap::prepareEntry($basedn, $mapiEntry);
    if (my $result = Torus::Driver::ldap::addEntry($entry)) {
      #my $error = 0;
      #$error = 1 if $result->code >= 0;
      #Torus::Driver::ldap::traceEntry($mapiEntry, $entry, { error => $error, prefix => $result->error }) if $TRACELEVEL >= 2;
      #undef $result;
      Torus::Driver::ldap::processResult($entry->{identifier}, $result);
    }
  }

}

sub startSync {

  my $self = shift;
  my $opts = shift;

  my $ldapLocator = Data::Storage::Locator->new(
    ldap => {
      type => "NetLDAP",
      dsn => "ldap:host=192.168.1.56;binddn='cn=root, dc=labnet, dc=de';pass=Geheim",
      #schema => 'BizWorks::BackendDbSchema',
      #classnames => [qw( SystemEvent LangText )],
      #classnames => [qw( SystemEvent Person Address LangText )],
      #test_availability => 1,
      #test_integrity => 1,
      #test_emptyness => 1,
      # TODO: re-enable this! (multiple-target-logging!)
      #logger => 1,
      want_transactions => 0,
      syncable => 1,
    },
  );
  my $ldapStorage = Data::Storage->new($ldapLocator);
  $ldapStorage->connect();

  my $mapiStorage;
  

  # create a new synchronization object
  my $sync = Data::Transfer::Sync->new();


  # resolve mapping metadata for syncing requested resource
    # if now mapping is given explicitely, try to use mapping named like "target"
    $opts->{mapping} ||= $opts->{target};
    # was: ($nodemapping and $propmapping were package-globals)
    # is: (both mappings are retrieved from mapping-module/package/object
    # resolve mapping module
    my $modulename = $self->{config}->{mappings}->{$opts->{mapping}};
    $opts->{mapping_module} = $modulename;

#print Dumper($opts);

  # mungle & check the options
  # TODO: refactor this and the innards.....

    #$opts = $sync->prepareSyncOptions($opts);
    if (!$sync->prepareOptions($opts)) {
      $logger->critical( __PACKAGE__ . "->startSync: 'Data::Transfer::Sync::prepareSyncOptions' failed.");
    }

#print Dumper($opts);
    
    if (!$sync->checkOptions($opts)) {
      $logger->critical( __PACKAGE__ . "->startSync: 'Data::Transfer::Sync::checkOptions' failed.");
      return;
    }

  # branch to execution path for special targets
  # detect for option 'handler' which could be a CODEref
    if ($opts->{handler} && ref $opts->{handler} eq 'CODE') {
      $logger->info( __PACKAGE__ . "->startSync: Running (special handler code - no generic sync!) on '$opts->{target}' with MODE $opts->{mode}, NODE $opts->{target_node}");
      $opts->{handler}->($self, $opts);
      return;
    }

  # determine write-protection(s) from options
    my @write_protects;
    push @write_protects, 'L' if $opts->{source_write_protect};
    push @write_protects, 'R' if $opts->{target_write_protect};
  
  # configure synchronization-object
  $sync->configure( 
    storages => {
      #L => $self->{bizWorks}->{$opts->{source}},
      L => $mapiStorage,
      #R => $self->{bizWorks}->{$opts->{target}},
      R => $ldapStorage,
    },
    id_authorities => [qw( L ) ],
    checksum_authorities => [qw( L ) ],
    write_protected => \@write_protects,
    verbose => 1,
  );
  
  # patch options
  $opts->{source} = "L:$opts->{source_node}" if $opts->{source_node};
  $opts->{target} = "R:$opts->{target_node}" if $opts->{target_node};

#print Dumper($opts);

  $sync->syncNodes($opts);

}

1;
